# -*- coding: utf-8 -*-
"""Host Controller"""

import transaction

from tg import expose, flash, require, url, request, redirect, validate
from tg.decorators import override_template
from pylons.i18n import ugettext as _, lazy_ugettext as l_
from tgext.admin.tgadminconfig import TGAdminConfig
from tgext.admin.controller import AdminController
from repoze.what import predicates
from sqlalchemy import and_

from mirrormanager.lib.base import BaseController
from mirrormanager.model import DBSession, metadata
from mirrormanager.controllers.error import ErrorController
from mirrormanager import model
from mirrormanager.controllers.secure import SecureController

from mirrormanager import model
from mirrormanager.model import *

from mirrormanager.controllers.forms import *
from mirrormanager.controllers.utils import *


host_form = HostForm("create_host_form", submit_text='Save Host')

class HostController(BaseController):
    allow_only = predicates.not_anonymous()

    def disabled_fields(self, host=None):
        disabled_fields = []
        if not predicates.in_group(admin_group):
            disabled_fields.append('admin_active')

        if host is not None:
            site = model.DBSession.query(Site).filter_by(id = host.site).one()
            if not site.is_siteadmin(request.identity['repoze.who.userid']):
                for a in ['user_active', 'private', 'robot_email']:
                    disabled_fields.append(a)
        return disabled_fields


    @expose(template="mirrormanager.templates.host")
    def new(self, siteid):
        try:
            site = model.DBSession.query(Site).filter_by(id = siteid).one()
        except Exception, er:
            print "Error:", er
            raise redirect(url("/"))
        submit_action = url("/host/create?siteid=%s" % siteid)
        return dict(form=host_form, values=None,
                    action=submit_action,
                    disabled_fields=self.disabled_fields(),
                    page_title="Create Host", site=site)

    @expose(template="mirrormanager.templates.host")
    @validate(form=host_form)
    #@error_handler(new)
    def create(self, siteid=None, tg_errors=None, name=None, 
        admin_active=None, user_active=None, country=None,
        bandwidth_int=None, private=None, internet2=None,
        internet2_clients=None, asn=None, asn_clients=None,
        robot_email=None, comment=None):
        if not predicates.in_group(admin_group) and bool(admin_active):
            admin_active = None
        site = model.DBSession.query(Site).filter_by(id = siteid).one()
        submit_action = url("/host/create?siteid=%s" % site.id)
        errordict = dict(form=host_form, values=None, 
                        action=submit_action,
                        disabled_fields=self.disabled_fields(),
                        page_title="Create Host", site=site)

        # handle the validation error
        if tg_errors:
            flash("Error creating Host: %s" % (createErrorString(tg_errors)))
            return errordict

        try:
            host = Host()
            host.name = name
            host.site = site.id
            host.admin_active = bool(admin_active)
            host.user_active = bool(user_active)
            host.country = country
            if bandwidth_int is not None and bandwidth_int != "" :
                host.bandwidth_int = int(bandwidth_int)
            host.private = bool(private)
            host.internet2 = bool(internet2)
            host.internet2_clients = bool(internet2_clients)
            host.asn = asn
            host.asn_clients = bool(asn_clients)
            host.robot_email = robot_email
            host.comment = comment
            model.DBSession.add(host)
            transaction.commit()
            site = model.DBSession.query(Site).filter_by(id = siteid).one()
            host = model.DBSession.query(Host).filter_by(name = name, site = site.id).one()
        except ValueError, er:
            print "Error", er
            flash("Error: One of the value given is not possible")
            return errordict
        except: # probably sqlite IntegrityError but we can't catch that for some reason... 
            flash("Error:Host %s already exists" % name)
            return errordict
        
        flash("Host created.")
        raise redirect(url("/host/%s" % host.id))


    @expose(template="mirrormanager.templates.host")
    def default(self, host):
        host = model.DBSession.query(Host).filter_by(id = host).one()
        site = model.DBSession.query(Site).filter_by(id = host.site).one()
        downstream_siteadmin_check(site, request.identity['repoze.who.userid'])
        host.categories = model.DBSession.query(HostCategory).filter_by(host = host.id).all()
        host.acl_ips = model.DBSession.query(HostAclIp).filter_by(host = host.id).all()
        host.netblocks = model.DBSession.query(HostNetblock).filter_by(host = host.id).all()
        host.countries_allowed = model.DBSession.query(HostCountryAllowed).filter_by(host = host.id).all()
        submit_action = url("/host/update/%s/" % host.id)
        return dict(form=host_form, values=host, action=submit_action,
                    disabled_fields=self.disabled_fields(host=host),
                    page_title="Host", site=site)

    @expose(template="mirrormanager.templates.host")
    @validate(form=host_form)
    #@error_handler()
    def update(self, host, tg_errors=None, name=None, 
        admin_active=None, user_active=None, country=None,
        bandwidth_int=None, private=None, internet2=None,
        internet2_clients=None, asn=None, asn_clients=None,
        robot_email=None, comment=None):

        host = model.DBSession.query(Host).filter_by(id = host).one()
        site = model.DBSession.query(Site).filter_by(id = host.site).one()
        siteadmin_check(site, request.identity['repoze.who.userid'])

        if tg_errors is not None:
            submit_action =url( "/host/update/%s" % host.id)
            flash("Error updating Host: %s" % createErrorString(tg_errors))
            return dict(form=host_form, values=host, action=submit_action,
                        disabled_fields=self.disabled_fields(host=host), page_title="Host", site=host.site)


        if not predicates.in_group(admin_group) and bool(admin_active):
            admin_active = None

        try:
            info = {'name':name, 'admin_active':bool(admin_active),
                'user_active':bool(user_active),
                'country':country, 'bandwidth_int':int(bandwidth_int),
                'private':bool(private), 'internet2':bool(internet2),
                'internet2_clients':bool(internet2_clients),
                'asn':asn, 'asn_clients':bool(asn_clients),
                'robot_email':robot_email, 'comment':comment }
        except ValueError, er:
            print "Error", er
            flash("Error: One of the value given is not possible")
            raise redirect(url("/host/%s" % host.id))

        model.DBSession.query(Host).update(info)
        flash("Host Updated")
        raise redirect(url("/host/%s" % host.id))

    @expose()
    def delete(self, host, **kwargs):
        host = model.DBSession.query(Host).filter_by(id = host).one()
        site = model.DBSession.query(Site).filter_by(id = host.site).one()
        siteadmin_check(site, request.identity['repoze.who.userid'])
        model.DBSession.delete(host)
        raise redirect(url("/site/%s" % site.id))
